@@ -7,10 +7,7 @@
 # the same terms as the Perl 5 programming language system itself.
 #
 package IO::Socket::Timeout;
-{
-  $IO::Socket::Timeout::VERSION = '0.27';
-}
-
+$IO::Socket::Timeout::VERSION = '0.28';
 use strict;
 use warnings;
 use Config;
@@ -77,8 +74,11 @@ BEGIN {
         # PerlIO::via::Timeout->READ calls sysread.
         my $_prevent_deep_recursion;
         *CORE::GLOBAL::sysread = sub {
-            $_prevent_deep_recursion || ! PerlIO::via::Timeout->_fh2prop($_[0])->{timeout_enabled}
-              and return CORE::sysread($_[0], $_[1], $_[2], $_[3]);
+            my $args_count = scalar(@_);
+            $_prevent_deep_recursion || ! (PerlIO::via::Timeout->_fh2prop($_[0]) || {})->{timeout_enabled}
+              and return (  $args_count == 4 ? CORE::sysread($_[0], $_[1], $_[2], $_[3])
+                          :                    CORE::sysread($_[0], $_[1], $_[2])
+                         );
             $_prevent_deep_recursion = 1;
             require PerlIO::via::Timeout;
             my $ret_val = PerlIO::via::Timeout->READ($_[1], $_[2], $_[0]);
@@ -98,8 +98,12 @@ BEGIN {
         # PerlIO::via::Timeout->WRITE calls syswrite.
         my $_prevent_deep_recursion;
         *CORE::GLOBAL::syswrite = sub {
-            $_prevent_deep_recursion || ! PerlIO::via::Timeout->_fh2prop($_[0])->{timeout_enabled}
-              and return CORE::syswrite($_[0], $_[1], $_[2], $_[3]);
+            my $args_count = scalar(@_);
+            $_prevent_deep_recursion || ! (PerlIO::via::Timeout->_fh2prop($_[0]) || {})->{timeout_enabled}
+              and return(   $args_count == 4 ? CORE::syswrite($_[0], $_[1], $_[2], $_[3])
+                          : $args_count == 3 ? CORE::syswrite($_[0], $_[1], $_[2])
+                          :                    CORE::syswrite($_[0], $_[1])
+                        );
             $_prevent_deep_recursion = 1;
             require PerlIO::via::Timeout;
             my $ret_val = PerlIO::via::Timeout->WRITE($_[1], $_[0]);
@@ -110,9 +114,7 @@ BEGIN {
 }
 
 package IO::Socket::Timeout::Role::SetSockOpt;
-{
-  $IO::Socket::Timeout::Role::SetSockOpt::VERSION = '0.27';
-}
+$IO::Socket::Timeout::Role::SetSockOpt::VERSION = '0.28';
 use Carp;
 use Socket;
 
@@ -170,9 +172,7 @@ sub _set_sock_opt {
 }
 
 package IO::Socket::Timeout::Role::PerlIO;
-{
-  $IO::Socket::Timeout::Role::PerlIO::VERSION = '0.27';
-}
+$IO::Socket::Timeout::Role::PerlIO::VERSION = '0.28';
 use PerlIO::via::Timeout;
 
 sub read_timeout    { goto &PerlIO::via::Timeout::read_timeout    }
@@ -195,7 +195,7 @@ IO::Socket::Timeout - IO::Socket with read/write timeout
 
 =head1 VERSION
 
-version 0.27
+version 0.28
 
 =head1 SYNOPSIS
 
